@@ -1,7 +1,7 @@
 /* GOB C Preprocessor
  * Copyright (C) 1999-2000 the Free Software Foundation.
  * Copyright (C) 2000 Eazel, Inc.
- * Copyright (C) 2001 George Lebl
+ * Copyright (C) 2001-2004 George Lebl
  *
  * Author: George Lebl
  *
@@ -334,6 +334,19 @@ check_signal_args (Class *c)
 				continue;
 
 			for (l = m->gtktypes; l != NULL; l = l->next) {
+				if (strcmp (l->data, "BOXED") == 0) {
+ 					error_printf (GOB_ERROR, m->line_no,
+ 						      "BOXED not allowed as "
+						      "a signal argument, use "
+ 						      "POINTER, or BOXED_*");
+ 					continue;
+				} else if (strcmp (l->data, "FLAGS") == 0) {
+					error_printf (GOB_ERROR, m->line_no,
+						      "FLAGS not allowed as "
+						      "a signal argument, use "
+						      "UINT");
+					continue;
+				}
 				if (get_cast (l->data, FALSE))
 					continue;
 				error_printf (GOB_ERROR, m->line_no,
@@ -465,6 +478,26 @@ check_func_arg_checks(Class *c)
 	}
 }
 
+void
+check_for_class_destructors (Class *c)
+{
+	GList *li;
+	for (li = c->nodes; li != NULL; li = li->next) {
+		Node *n = li->data;
+		if (n->type == VARIABLE_NODE) {
+			Variable *v = (Variable *)n;
+			if (v->destructor != NULL &&
+			    v->scope == CLASS_SCOPE) {
+				error_print (GOB_WARN, v->line_no,
+					     "classwide members cannot have "
+					     "destructors since the classes "
+					     "are static and never get "
+					     "destroyed anyway");
+			}
+		}
+	}
+}
+
 int
 count_signals(Class *c)
 {
@@ -601,7 +634,8 @@ count_unreftors (Class *c)
 		if (n->type == VARIABLE_NODE) {
 			Variable *v = (Variable *)n;
 			if (v->destructor != NULL &&
-			    v->destructor_unref)
+			    v->destructor_unref &&
+			    v->scope != CLASS_SCOPE)
 				num++;
 		}
 	}
@@ -618,7 +652,8 @@ count_destructors (Class *c)
 		if (n->type == VARIABLE_NODE) {
 			Variable *v = (Variable *)n;
 			if (v->destructor != NULL &&
-			    ! v->destructor_unref)
+			    ! v->destructor_unref &&
+			    v->scope != CLASS_SCOPE)
 				num++;
 		}
 	}
@@ -641,6 +676,22 @@ count_initializers (Class *c)
 	return num;
 }
 
+int
+count_glade_widgets (Class *c)
+{
+	int num = 0;
+	GList *li;
+	for (li = c->nodes; li != NULL; li = li->next) {
+		Node *n = li->data;
+		if (n->type == VARIABLE_NODE) {
+			Variable *v = (Variable *)n;
+			if (v->glade_widget)
+				num++;
+		}
+	}
+	return num;
+}
+
 gboolean
 find_get_type (Class *c)
 {
